/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#include "swcommands.h"
#include <svx/svxcommands.h>
#include <sfx2/sfxcommands.h>

#define HID_BIB_BASE                                            "SW_HID_BIB_BASE"
#define HID_MERGE_PRINTMONITOR                                  "SW_HID_MERGE_PRINTMONITOR"
#define HID_FLD_INPUT                                           "SW_HID_FLD_INPUT"
#define HID_DOCINFO_EDT                                         "SW_HID_DOCINFO_EDT"
#define HID_PASSWD                                              "SW_HID_PASSWD"
#define HID_FORMAT_COLUMN                                       "SW_HID_FORMAT_COLUMN"
#define HID_CONFIG_SAVE                                         "SW_HID_CONFIG_SAVE"
#define HID_NUM_NAMES                                           "SW_HID_NUM_NAMES"

#define HID_FORMEDT_CONTENT                                     "SW_HID_FORMEDT_CONTENT"
#define HID_FORMEDT_USER                                        "SW_HID_FORMEDT_USER"
#define HID_FORMEDT_INDEX                                       "SW_HID_FORMEDT_INDEX"

#define HID_SCRL_PAGEUP                                         "SW_HID_SCRL_PAGEUP"
#define HID_SCRL_PAGEDOWN                                       "SW_HID_SCRL_PAGEDOWN"

#define HID_EDIT_WIN                                            "SW_HID_EDIT_WIN"

#define HID_INSERT_CTRL                                         "SW_HID_INSERT_CTRL" // TbxControl Einfuegen
#define HID_INSERT_FIELD_CTRL                                   "SW_HID_INSERT_FIELD_CTRL"

#define HID_SOURCEVIEW                                          "SW_HID_SOURCEVIEW"
#define HID_AUTOFORMAT_CLB                                      "SW_HID_AUTOFORMAT_CLB"

#define HID_SCRL_NAVI                                           "SW_HID_SCRL_NAVI"
#define HID_NAVI_DRAG_HYP                                       "SW_HID_NAVI_DRAG_HYP"
#define HID_NAVI_DRAG_LINK                                      "SW_HID_NAVI_DRAG_LINK"
#define HID_NAVI_DRAG_COPY                                      "SW_HID_NAVI_DRAG_COPY"
#define HID_NAVI_OUTLINES                                       "SW_HID_NAVI_OUTLINES"

#define HID_AUTOFORMAT_EXEC                                     "SW_HID_AUTOFORMAT_EXEC"
#define HID_AUTOFORMAT_CLOSE                                    "SW_HID_AUTOFORMAT_CLOSE"

#define HID_PAGEPREVIEW                                         "SW_HID_PAGEPREVIEW"
#define HID_SOURCE_EDITWIN                                      "SW_HID_SOURCE_EDITWIN"

// Dialog Help-IDs

#define HID_INSERT_CHART                                        "SW_HID_INSERT_CHART"
#define HID_NAVIGATOR_TREELIST                                  "SW_HID_NAVIGATOR_TREELIST"
#define HID_DLG_CAPTION                                         "SW_HID_DLG_CAPTION"
#define HID_DLG_GLOSS_DECIDE                                    "SW_HID_DLG_GLOSS_DECIDE"
#define HID_NAVIGATOR_TOOLBOX                                   "SW_HID_NAVIGATOR_TOOLBOX"
#define HID_NAVIGATOR_LISTBOX                                   "SW_HID_NAVIGATOR_LISTBOX"
#define HID_VS_SINGLENUM                                        "SW_HID_VS_SINGLENUM"
#define HID_VS_NUM                                              "SW_HID_VS_NUM"
#define HID_VS_BULLET                                           "SW_HID_VS_BULLET"
#define HID_VS_NUMBMP                                           "SW_HID_VS_NUMBMP"
#define HID_INSERT_ABSTRACT                                     "SW_HID_INSERT_ABSTRACT"
#define HID_NAVI_TBX1                                           "SW_HID_NAVI_TBX1"
#define HID_NAVI_TBX2                                           "SW_HID_NAVI_TBX2"
#define HID_NAVI_TBX3                                           "SW_HID_NAVI_TBX3"
#define HID_NAVI_TBX4                                           "SW_HID_NAVI_TBX4"
#define HID_NAVI_TBX5                                           "SW_HID_NAVI_TBX5"
#define HID_NAVI_TBX6                                           "SW_HID_NAVI_TBX6"
#define HID_NAVI_TBX7                                           "SW_HID_NAVI_TBX7"
#define HID_NAVI_TBX8                                           "SW_HID_NAVI_TBX8"
#define HID_NAVI_TBX9                                           "SW_HID_NAVI_TBX9"
#define HID_NAVI_TBX10                                          "SW_HID_NAVI_TBX10"
#define HID_NAVI_TBX11                                          "SW_HID_NAVI_TBX11"
#define HID_NAVI_TBX12                                          "SW_HID_NAVI_TBX12"
#define HID_NAVI_TBX13                                          "SW_HID_NAVI_TBX13"
#define HID_NAVI_TBX14                                          "SW_HID_NAVI_TBX14"
#define HID_NAVI_TBX15                                          "SW_HID_NAVI_TBX15"
#define HID_NAVI_VS                                             "SW_HID_NAVI_VS"
#define HID_NUM_FORMAT_BTN                                      "SW_HID_NUM_FORMAT_BTN"
#define HID_PPREV_ZOOM                                          "SW_HID_PPREV_ZOOM"
#define HID_NAVI_TBX16                                          "SW_HID_NAVI_TBX16"
#define HID_LTEMPL_TEXT                                         "SW_HID_LTEMPL_TEXT"
#define HID_LTEMPL_FRAME                                        "SW_HID_LTEMPL_FRAME"
#define HID_LTEMPL_PAGE                                         "SW_HID_LTEMPL_PAGE"
#define HID_LTEMPL_OVERRIDE                                     "SW_HID_LTEMPL_OVERRIDE"
#define HID_NAVI_TBX17                                          "SW_HID_NAVI_TBX17"
#define HID_NAVI_TBX18                                          "SW_HID_NAVI_TBX18"
#define HID_NAVI_TBX19                                          "SW_HID_NAVI_TBX19"
#define HID_NAVI_TBX20                                          "SW_HID_NAVI_TBX20"
#define HID_NAVI_TBX21                                          "SW_HID_NAVI_TBX21"
#define HID_NAVI_TBX22                                          "SW_HID_NAVI_TBX22"
#define HID_NAVI_TBX23                                          "SW_HID_NAVI_TBX23"
#define HID_NAVI_TBX24                                          "SW_HID_NAVI_TBX24"
#define HID_NAVIGATOR_GLOBAL_TOOLBOX                            "SW_HID_NAVIGATOR_GLOBAL_TOOLBOX"
#define HID_NAVIGATOR_GLOB_TREELIST                             "SW_HID_NAVIGATOR_GLOB_TREELIST"
#define HID_GLOS_GROUP_TREE                                     "SW_HID_GLOS_GROUP_TREE"
#define HID_GLBLTREE_UPDATE                                     "SW_HID_GLBLTREE_UPDATE"
#define HID_GLBLTREE_INSERT                                     "SW_HID_GLBLTREE_INSERT"
#define HID_GLBLTREE_EDIT                                       "SW_HID_GLBLTREE_EDIT"
#define HID_GLBLTREE_DEL                                        "SW_HID_GLBLTREE_DEL"
#define HID_GLBLTREE_INS_IDX                                    "SW_HID_GLBLTREE_INS_IDX"
#define HID_GLBLTREE_INS_CNTIDX                                 "SW_HID_GLBLTREE_INS_CNTIDX"
#define HID_GLBLTREE_INS_USRIDX                                 "SW_HID_GLBLTREE_INS_USRIDX"
#define HID_GLBLTREE_INS_FILE                                   "SW_HID_GLBLTREE_INS_FILE"
#define HID_GLBLTREE_INS_NEW_FILE                               "SW_HID_GLBLTREE_INS_NEW_FILE"
#define HID_GLBLTREE_INS_TEXT                                   "SW_HID_GLBLTREE_INS_TEXT"
#define HID_GLBLTREE_UPD_SEL                                    "SW_HID_GLBLTREE_UPD_SEL"
#define HID_GLBLTREE_UPD_IDX                                    "SW_HID_GLBLTREE_UPD_IDX"
#define HID_GLBLTREE_UPD_LINK                                   "SW_HID_GLBLTREE_UPD_LINK"
#define HID_GLBLTREEUPD_ALL                                     "SW_HID_GLBLTREEUPD_ALL"
#define HID_NUM_OUTL_NAMED_NUMS                                 "SW_HID_NUM_OUTL_NAMED_NUMS"
#define HID_NUM_OUTL_NUM_SAVEAS                                 "SW_HID_NUM_OUTL_NUM_SAVEAS"
#define HID_NUM_NAMED_NUMS                                      "SW_HID_NUM_NAMED_NUMS"
#define HID_NAVI_CONTENT                                        "SW_HID_NAVI_CONTENT"
#define HID_NAVI_GLOBAL                                         "SW_HID_NAVI_GLOBAL"
#define HID_LTEMPL_NUMBERING                                    "SW_HID_LTEMPL_NUMBERING"
#define HID_FLDVAR_APPLY                                        "SW_HID_FLDVAR_APPLY"
#define HID_FLDVAR_DELETE                                       "SW_HID_FLDVAR_DELETE"
#define HID_FLDEDT_ADDRESS                                      "SW_HID_FLDEDT_ADDRESS"
#define HID_SORT_ACTION                                         "SW_HID_SORT_ACTION"
#define HID_SORT_AUTHOR                                         "SW_HID_SORT_AUTHOR"
#define HID_SORT_DATE                                           "SW_HID_SORT_DATE"
#define HID_SORT_COMMENT                                        "SW_HID_SORT_COMMENT"
#define HID_SW_SORT_POSITION                                    "SW_HID_SW_SORT_POSITION"
#define HID_SYNC_BTN                                            "SW_HID_SYNC_BTN"
#define HID_EDIT_COMMENT                                        "SW_HID_EDIT_COMMENT"
#define HID_DLG_SEQUENCE_OPTION                                 "SW_HID_DLG_SEQUENCE_OPTION"
#define HID_DLG_FLDEDT_NEXT                                     "SW_HID_DLG_FLDEDT_NEXT"
#define HID_DLG_FLDEDT_PREV                                     "SW_HID_DLG_FLDEDT_PREV"
#define HID_DLG_FLDEDT_ADDRESS                                  "SW_HID_DLG_FLDEDT_ADDRESS"

#define HID_FILEDLG_CHARDLG                                     "SW_HID_FILEDLG_CHARDLG"
#define HID_FILEDLG_LOADTEMPLATE                                "SW_HID_FILEDLG_LOADTEMPLATE"
#define HID_FILEDLG_MAILMRGE1                                   "SW_HID_FILEDLG_MAILMRGE1"
#define HID_FILEDLG_MAILMRGE2                                   "SW_HID_FILEDLG_MAILMRGE2"
#define HID_FILEDLG_FRMPAGE                                     "SW_HID_FILEDLG_FRMPAGE"
#define HID_FILEDLG_SRCVIEW                                     "SW_HID_FILEDLG_SRCVIEW"
#define HID_FILEDLG_WIZDOKU                                     "SW_HID_FILEDLG_WIZDOKU"

#define HID_REDLINE_AUTOFMT_ACCEPT                              "SW_HID_REDLINE_AUTOFMT_ACCEPT"
#define HID_MULTI_TOX_DLG                                       "SW_HID_MULTI_TOX_DLG"
#define HID_ASCII_FILTER                                        "SW_HID_ASCII_FILTER"
#define HID_GLBLTREE_EDIT_LINK                                  "SW_HID_GLBLTREE_EDIT_LINK"
#define HID_FORMAT_NAME_OBJECT_NAME                             "SW_HID_FORMAT_NAME_OBJECT_NAME"
#define HID_REFSELECTION_TOOLTIP                                "SW_HID_REFSELECTION_TOOLTIP"    // #i83479#


// TabPage Help-IDs

#define HID_DROPCAPS                                            "SW_HID_DROPCAPS"
#define HID_FRM_EXT                                             "SW_HID_FRM_EXT"
#define HID_FRM_STD                                             "SW_HID_FRM_STD"
#define HID_GRF_EXT                                             "SW_HID_GRF_EXT"
#define HID_COLUMN                                              "SW_HID_COLUMN"
#define HID_DOC_STAT                                            "SW_HID_DOC_STAT"
#define HID_FOOTNOTE_PAGE                                       "SW_HID_FOOTNOTE_PAGE"
#define HID_OPTPRINT_PAGE                                       "SW_HID_OPTPRINT_PAGE"
#define HID_FORMAT_TABLE                                        "SW_HID_FORMAT_TABLE"
#define HID_TABLE_COLUMN                                        "SW_HID_TABLE_COLUMN"
#define HID_LAB_LAB                                             "SW_HID_LAB_LAB"
#define HID_LAB_PRT                                             "SW_HID_LAB_PRT"
#define HID_ENV_PRT                                             "SW_HID_ENV_PRT"
#define HID_LAB_FMT                                             "SW_HID_LAB_FMT"
#define HID_ENV_ENV                                             "SW_HID_ENV_ENV"
#define HID_ENV_FMT                                             "SW_HID_ENV_FMT"
#define HID_FRM_URL                                             "SW_HID_FRM_URL"
#define HID_CONTENT_OPT                                         "SW_HID_CONTENT_OPT"
#define HID_STD_FONT                                            "SW_HID_STD_FONT"
#define HID_COND_COLL                                           "SW_HID_COND_COLL"
#define HID_FRM_ADD                                             "SW_HID_FRM_ADD"
#define HID_TABLE_TEXTFLOW                                      "SW_HID_TABLE_TEXTFLOW"
#define HID_OPTTABLE_PAGE                                       "SW_HID_OPTTABLE_PAGE"

#define HID_FLD_DB                                              "SW_HID_FLD_DB"
#define HID_FLD_DOKINF                                          "SW_HID_FLD_DOKINF"
#define HID_FLD_VAR                                             "SW_HID_FLD_VAR"
#define HID_FLD_DOK                                             "SW_HID_FLD_DOK"
#define HID_FLD_FUNC                                            "SW_HID_FLD_FUNC"
#define HID_FLD_REF                                             "SW_HID_FLD_REF"
#define HID_OPTSHDWCRSR                                         "SW_HID_OPTSHDWCRSR"
#define HID_NUMPARA                                             "SW_HID_NUMPARA"
#define HID_REDLINE_OPT                                         "SW_HID_REDLINE_OPT"
#define HID_REDLINE_ACCEPT                                      "SW_HID_REDLINE_ACCEPT"
#define HID_EDIT_FLD_DB                                         "SW_HID_EDIT_FLD_DB"
#define HID_EDIT_FLD_DOKINF                                     "SW_HID_EDIT_FLD_DOKINF"
#define HID_EDIT_FLD_VAR                                        "SW_HID_EDIT_FLD_VAR"
#define HID_EDIT_FLD_DOK                                        "SW_HID_EDIT_FLD_DOK"
#define HID_EDIT_FLD_FUNC                                       "SW_HID_EDIT_FLD_FUNC"
#define HID_EDIT_FLD_REF                                        "SW_HID_EDIT_FLD_REF"
#define HID_REDLINE_CTRL                                        "SW_HID_REDLINE_CTRL"
#define HID_OPTCAPTION_PAGE                                     "SW_HID_OPTCAPTION_PAGE"
#define HID_OPTLOAD_PAGE                                        "SW_HID_OPTLOAD_PAGE"
#define HID_INSERT_SECTION_PAGE                                 "SW_HID_INSERT_SECTION_PAGE"
#define HID_INSERT_SECTION_DLG                                  "SW_HID_INSERT_SECTION_DLG"
#define HID_SECTION_PROPERTIES_DLG                              "SW_HID_SECTION_PROPERTIES_DLG"
#define HID_RENAME_XNAMED                                       "SW_HID_RENAME_XNAMED"
#define HID_TP_TOX_SELECT                                       "SW_HID_TP_TOX_SELECT"
#define HID_TP_TOX_ENTRY                                        "SW_HID_TP_TOX_ENTRY"
#define HID_TOX_STYLES_PAGE                                     "SW_HID_TOX_STYLES_PAGE"
#define HID_TP_VISITING_CARDS                                   "SW_HID_TP_VISITING_CARDS"
#define HID_DLG_ADD_IDX_STYLES                                  "SW_HID_DLG_ADD_IDX_STYLES"
#define HID_ADD_STYLES_TLB                                      "SW_HID_ADD_STYLES_TLB"
#define HID_OLE_CHECKLB                                         "SW_HID_OLE_CHECKLB"
#define HID_DLG_CREATE_AUTH_ENTRY                               "SW_HID_DLG_CREATE_AUTH_ENTRY"
#define HID_TP_PRIVATE_DATA                                     "SW_HID_TP_PRIVATE_DATA"

#define HID_TP_BUSINESS_DATA                                    "SW_HID_TP_BUSINESS_DATA"
#define HID_SECTION_FTNENDNOTES_PAGE                            "SW_HID_SECTION_FTNENDNOTES_PAGE"
#define HID_TEXTGRID_PAGE                                       "SW_HID_TEXTGRID_PAGE"
#define HID_OPTCOMPATIBILITY_PAGE                               "SW_HID_OPTCOMPATIBILITY_PAGE"
#define HID_COMPATIBILITY_OPTIONS_BOX                           "SW_HID_COMPATIBILITY_OPTIONS_BOX"
#define HID_COMPARISON_OPT				                        "SW_HID_COMPARISON_OPT"

// AutoPilot Help-IDs

#define HID_LETTER_PAGE1                                        "SW_HID_LETTER_PAGE1"
#define HID_LETTER_PAGE2                                        "SW_HID_LETTER_PAGE2"
#define HID_LETTER_PAGE3                                        "SW_HID_LETTER_PAGE3"
#define HID_LETTER_PAGE4                                        "SW_HID_LETTER_PAGE4"
#define HID_LETTER_PAGE5                                        "SW_HID_LETTER_PAGE5"
#define HID_LETTER_PAGE6                                        "SW_HID_LETTER_PAGE6"
#define HID_LETTER_PAGE7                                        "SW_HID_LETTER_PAGE7"
#define HID_LETTER_PAGE8                                        "SW_HID_LETTER_PAGE8"
#define HID_LETTER_PAGE9                                        "SW_HID_LETTER_PAGE9"

#define HID_FAX_PAGE1                                           "SW_HID_FAX_PAGE1"
#define HID_FAX_PAGE2                                           "SW_HID_FAX_PAGE2"
#define HID_FAX_PAGE3                                           "SW_HID_FAX_PAGE3"
#define HID_FAX_PAGE4                                           "SW_HID_FAX_PAGE4"
#define HID_FAX_PAGE5                                           "SW_HID_FAX_PAGE5"
#define HID_FAX_PAGE6                                           "SW_HID_FAX_PAGE6"
#define HID_FAX_PAGE7                                           "SW_HID_FAX_PAGE7"
#define HID_FAX_PAGE8                                           "SW_HID_FAX_PAGE8"

#define HID_MEMO_PAGE1                                          "SW_HID_MEMO_PAGE1"
#define HID_MEMO_PAGE2                                          "SW_HID_MEMO_PAGE2"
#define HID_MEMO_PAGE3                                          "SW_HID_MEMO_PAGE3"
#define HID_MEMO_PAGE4                                          "SW_HID_MEMO_PAGE4"
#define HID_MEMO_PAGE5                                          "SW_HID_MEMO_PAGE5"

#define HID_AGENDA_PAGE1                                        "SW_HID_AGENDA_PAGE1"
#define HID_AGENDA_PAGE2                                        "SW_HID_AGENDA_PAGE2"
#define HID_AGENDA_PAGE3                                        "SW_HID_AGENDA_PAGE3"
#define HID_AGENDA_PAGE4                                        "SW_HID_AGENDA_PAGE4"
#define HID_AGENDA_PAGE5                                        "SW_HID_AGENDA_PAGE5"
#define HID_AGENDA_PAGE6                                        "SW_HID_AGENDA_PAGE6"


// HelpIds for Menu

#define HID_MN_SUB_TBLROW                                       "SW_HID_MN_SUB_TBLROW"
#define HID_MN_SUB_TBLCOL                                       "SW_HID_MN_SUB_TBLCOL"
#define HID_MN_SUB_FIELD                                        "SW_HID_MN_SUB_FIELD"
#define HID_MN_SUB_GRAPHIC                                      "SW_HID_MN_SUB_GRAPHIC"
#define HID_MN_SUB_TEMPLATES                                    "SW_HID_MN_SUB_TEMPLATES"
#define HID_MN_SUB_ARRANGE                                      "SW_HID_MN_SUB_ARRANGE"
#define HID_MN_SUB_SPELLING                                     "SW_HID_MN_SUB_SPELLING"
#define HID_MN_SUB_MIRROR                                       "SW_HID_MN_SUB_MIRROR"
#define HID_MN_SUB_ALIGN                                        "SW_HID_MN_SUB_ALIGN"

#define HID_MN_CALC_PHD                                         "SW_HID_MN_CALC_PHD"
#define HID_MN_CALC_SQRT                                        "SW_HID_MN_CALC_SQRT"
#define HID_MN_CALC_OR                                          "SW_HID_MN_CALC_OR"
#define HID_MN_CALC_XOR                                         "SW_HID_MN_CALC_XOR"
#define HID_MN_CALC_AND                                         "SW_HID_MN_CALC_AND"
#define HID_MN_CALC_NOT                                         "SW_HID_MN_CALC_NOT"
#define HID_MN_CALC_EQ                                          "SW_HID_MN_CALC_EQ"
#define HID_MN_CALC_NEQ                                         "SW_HID_MN_CALC_NEQ"
#define HID_MN_CALC_LEQ                                         "SW_HID_MN_CALC_LEQ"
#define HID_MN_CALC_GEQ                                         "SW_HID_MN_CALC_GEQ"
#define HID_MN_CALC_LES                                         "SW_HID_MN_CALC_LES"
#define HID_MN_CALC_GRE                                         "SW_HID_MN_CALC_GRE"
#define HID_MN_CALC_SUM                                         "SW_HID_MN_CALC_SUM"
#define HID_MN_CALC_MEAN                                        "SW_HID_MN_CALC_MEAN"
#define HID_MN_CALC_MIN                                         "SW_HID_MN_CALC_MIN"
#define HID_MN_CALC_MAX                                         "SW_HID_MN_CALC_MAX"
#define HID_MN_CALC_SIN                                         "SW_HID_MN_CALC_SIN"
#define HID_MN_CALC_COS                                         "SW_HID_MN_CALC_COS"
#define HID_MN_CALC_TAN                                         "SW_HID_MN_CALC_TAN"
#define HID_MN_CALC_ASIN                                        "SW_HID_MN_CALC_ASIN"
#define HID_MN_CALC_ACOS                                        "SW_HID_MN_CALC_ACOS"
#define HID_MN_CALC_ATAN                                        "SW_HID_MN_CALC_ATAN"
#define HID_MN_CALC_POW                                         "SW_HID_MN_CALC_POW"
#define HID_MN_CALC_LISTSEP                                     "SW_HID_MN_CALC_LISTSEP"
#define HID_MN_POP_OPS                                          "SW_HID_MN_POP_OPS"
#define HID_MN_POP_STATISTICS                                   "SW_HID_MN_POP_STATISTICS"
#define HID_MN_POP_FUNC                                         "SW_HID_MN_POP_FUNC"
#define HID_MN_CALC_ROUND                                       "SW_HID_MN_CALC_ROUND"

#define HID_MN_READONLY_SAVEGRAPHIC                             "SW_HID_MN_READONLY_SAVEGRAPHIC"
#define HID_MN_READONLY_GRAPHICTOGALLERY                        "SW_HID_MN_READONLY_GRAPHICTOGALLERY"
#define HID_MN_READONLY_TOGALLERYLINK                           "SW_HID_MN_READONLY_TOGALLERYLINK"
#define HID_MN_READONLY_TOGALLERYCOPY                           "SW_HID_MN_READONLY_TOGALLERYCOPY"
#define HID_MN_READONLY_SAVEBACKGROUND                          "SW_HID_MN_READONLY_SAVEBACKGROUND"
#define HID_MN_READONLY_BACKGROUNDTOGALLERY                     "SW_HID_MN_READONLY_BACKGROUNDTOGALLERY"
#define HID_MN_READONLY_COPYLINK                                "SW_HID_MN_READONLY_COPYLINK"
#define HID_MN_READONLY_COPYGRAPHIC                             "SW_HID_MN_READONLY_COPYGRAPHIC"
#define HID_MN_READONLY_LOADGRAPHIC                             "SW_HID_MN_READONLY_LOADGRAPHIC"
#define HID_MN_READONLY_GRAPHICOFF                              "SW_HID_MN_READONLY_GRAPHICOFF"
#define HID_MN_READONLY_PLUGINOFF                               "SW_HID_MN_READONLY_PLUGINOFF"

#define HID_MD_GLOS_DEFINE                                      "SW_HID_MD_GLOS_DEFINE"
#define HID_MD_GLOS_REPLACE                                     "SW_HID_MD_GLOS_REPLACE"
#define HID_MD_GLOS_RENAME                                      "SW_HID_MD_GLOS_RENAME"
#define HID_MD_GLOS_DELETE                                      "SW_HID_MD_GLOS_DELETE"
#define HID_MD_GLOS_EDIT                                        "SW_HID_MD_GLOS_EDIT"
#define HID_MD_GLOS_MACRO                                       "SW_HID_MD_GLOS_MACRO"
#define HID_LINGU_ADD_WORD                                      "SW_HID_LINGU_ADD_WORD"
#define HID_LINGU_IGNORE_WORD                                   "SW_HID_LINGU_IGNORE_WORD"
#define HID_LINGU_SPELLING_DLG                                  "SW_HID_LINGU_SPELLING_DLG"
#define HID_LINGU_AUTOCORR                                      "SW_HID_LINGU_AUTOCORR"
#define HID_LINGU_REPLACE                                       "SW_HID_LINGU_REPLACE"
#define HID_MD_GLOS_CATEGORY                                    "SW_HID_MD_GLOS_CATEGORY"
#define HID_REGION_TREE                                         "SW_HID_REGION_TREE"
#define HID_LINGU_WORD_LANGUAGE                                 "SW_HID_LINGU_WORD_LANGUAGE"
#define HID_LINGU_PARA_LANGUAGE                                 "SW_HID_LINGU_PARA_LANGUAGE"
#define HID_MD_GLOS_DEFINE_TEXT                                 "SW_HID_MD_GLOS_DEFINE_TEXT"
#define HID_MD_COPY_TO_CLIPBOARD                                "SW_HID_MD_COPY_TO_CLIPBOARD"
#define HID_MD_GLOS_IMPORT                                      "SW_HID_MD_GLOS_IMPORT"
#define HID_SMARTTAG_MAIN                                       "SW_HID_SMARTTAG_MAIN"    // SMARTTAGS
#define HID_LINGU_IGNORE_SELECTION                              "SW_HID_LINGU_IGNORE_SELECTION"    // grammar check context menu

// More Help-IDs
#define HID_OPTCAPTION_CLB                                      "SW_HID_OPTCAPTION_CLB"
#define HID_EDIT_FORMULA                                        "SW_HID_EDIT_FORMULA"
#define HID_INSERT_FILE                                         "SW_HID_INSERT_FILE"
#define HID_FORMAT_PAGE                                         "SW_HID_FORMAT_PAGE"
#define HID_RENAME_GLOSSARY                                     "SW_HID_RENAME_GLOSSARY"
#define HID_CONFIG_MENU                                         "SW_HID_CONFIG_MENU"
#define HID_NAVIGATION_PI                                       "SW_HID_NAVIGATION_PI"
#define HID_ENVELOP_PRINT                                       "SW_HID_ENVELOP_PRINT"
#define HID_NAVIGATION_IMGBTN                                   "SW_HID_NAVIGATION_IMGBTN"
#define HID_ENVELOP_INSERT                                      "SW_HID_ENVELOP_INSERT"

#define HID_TEXT_TOOLBOX                                        "SW_HID_TEXT_TOOLBOX"
#define HID_TABLE_TOOLBOX                                       "SW_HID_TABLE_TOOLBOX"
#define HID_FRAME_TOOLBOX                                       "SW_HID_FRAME_TOOLBOX"
#define HID_GRAFIK_TOOLBOX                                      "SW_HID_GRAFIK_TOOLBOX"
#define HID_OLE_TOOLBOX                                         "SW_HID_OLE_TOOLBOX"
#define HID_DRAW_TOOLBOX                                        "SW_HID_DRAW_TOOLBOX"
#define HID_BEZIER_TOOLBOX                                      "SW_HID_BEZIER_TOOLBOX"
#define HID_DRAW_TEXT_TOOLBOX                                   "SW_HID_DRAW_TEXT_TOOLBOX"
#define HID_NUM_TOOLBOX                                         "SW_HID_NUM_TOOLBOX"

#define HID_LABEL_INSERT                                        "SW_HID_LABEL_INSERT"
#define HID_CALC_TOOLBOX                                        "SW_HID_CALC_TOOLBOX"
#define HID_PVIEW_TOOLBOX                                       "SW_HID_PVIEW_TOOLBOX"

#define HID_FIELD_INSERT                                        "SW_HID_FIELD_INSERT"
#define HID_FIELD_CLOSE                                         "SW_HID_FIELD_CLOSE"

#define HID_COND_COLL_TABLIST                                   "SW_HID_COND_COLL_TABLIST"

#define HID_DB_SELECTION_TLB                                    "SW_HID_DB_SELECTION_TLB"
#define HID_FIELD_DINF_TYPE                                     "SW_HID_FIELD_DINF_TYPE"

#define HID_SELECT_TEMPLATE                                     "SW_HID_SELECT_TEMPLATE"

#define HID_AP_INSERT_DB_SEL                                    "SW_HID_AP_INSERT_DB_SEL"
#define HID_OUTLINE_FORM                                        "SW_HID_OUTLINE_FORM"
#define HID_NUM_RESET                                           "SW_HID_NUM_RESET"

#define HID_AUTOFORMAT_REJECT                                   "SW_HID_AUTOFORMAT_REJECT"
#define HID_AUTOFORMAT_ACCEPT                                   "SW_HID_AUTOFORMAT_ACCEPT"
#define HID_AUTOFORMAT_EDIT_CHG                                 "SW_HID_AUTOFORMAT_EDIT_CHG"

//insert index/entries/Pattern buttons and edits
#define HID_TOX_ENTRY_BUTTON                                    "SW_HID_TOX_ENTRY_BUTTON"
#define HID_TOX_ENTRY_EDIT                                      "SW_HID_TOX_ENTRY_EDIT"

#define HID_DLG_NEW_USER_IDX                                    "SW_HID_DLG_NEW_USER_IDX"
#define HID_TOKEN_WINDOW                                        "SW_HID_TOKEN_WINDOW"

#define HID_AUTH_FIELD_IDENTIFIER                               "SW_HID_AUTH_FIELD_IDENTIFIER"
#define HID_AUTH_FIELD_AUTHORITY_TYPE                           "SW_HID_AUTH_FIELD_AUTHORITY_TYPE"
#define HID_AUTH_FIELD_ADDRESS                                  "SW_HID_AUTH_FIELD_ADDRESS"
#define HID_AUTH_FIELD_ANNOTE                                   "SW_HID_AUTH_FIELD_ANNOTE"
#define HID_AUTH_FIELD_AUTHOR                                   "SW_HID_AUTH_FIELD_AUTHOR"
#define HID_AUTH_FIELD_BOOKTITLE                                "SW_HID_AUTH_FIELD_BOOKTITLE"
#define HID_AUTH_FIELD_CHAPTER                                  "SW_HID_AUTH_FIELD_CHAPTER"
#define HID_AUTH_FIELD_EDITION                                  "SW_HID_AUTH_FIELD_EDITION"
#define HID_AUTH_FIELD_EDITOR                                   "SW_HID_AUTH_FIELD_EDITOR"
#define HID_AUTH_FIELD_HOWPUBLISHED                             "SW_HID_AUTH_FIELD_HOWPUBLISHED"
#define HID_AUTH_FIELD_INSTITUTION                              "SW_HID_AUTH_FIELD_INSTITUTION"
#define HID_AUTH_FIELD_JOURNAL                                  "SW_HID_AUTH_FIELD_JOURNAL"
#define HID_AUTH_FIELD_MONTH                                    "SW_HID_AUTH_FIELD_MONTH"
#define HID_AUTH_FIELD_NOTE                                     "SW_HID_AUTH_FIELD_NOTE"
#define HID_AUTH_FIELD_NUMBER                                   "SW_HID_AUTH_FIELD_NUMBER"
#define HID_AUTH_FIELD_ORGANIZATIONS                            "SW_HID_AUTH_FIELD_ORGANIZATIONS"
#define HID_AUTH_FIELD_PAGES                                    "SW_HID_AUTH_FIELD_PAGES"
#define HID_AUTH_FIELD_PUBLISHER                                "SW_HID_AUTH_FIELD_PUBLISHER"
#define HID_AUTH_FIELD_SCHOOL                                   "SW_HID_AUTH_FIELD_SCHOOL"
#define HID_AUTH_FIELD_SERIES                                   "SW_HID_AUTH_FIELD_SERIES"
#define HID_AUTH_FIELD_TITLE                                    "SW_HID_AUTH_FIELD_TITLE"
#define HID_AUTH_FIELD_REPORT_TYPE                              "SW_HID_AUTH_FIELD_REPORT_TYPE"
#define HID_AUTH_FIELD_VOLUME                                   "SW_HID_AUTH_FIELD_VOLUME"
#define HID_AUTH_FIELD_YEAR                                     "SW_HID_AUTH_FIELD_YEAR"
#define HID_AUTH_FIELD_URL                                      "SW_HID_AUTH_FIELD_URL"
#define HID_AUTH_FIELD_CUSTOM1                                  "SW_HID_AUTH_FIELD_CUSTOM1"
#define HID_AUTH_FIELD_CUSTOM2                                  "SW_HID_AUTH_FIELD_CUSTOM2"
#define HID_AUTH_FIELD_CUSTOM3                                  "SW_HID_AUTH_FIELD_CUSTOM3"
#define HID_AUTH_FIELD_CUSTOM4                                  "SW_HID_AUTH_FIELD_CUSTOM4"
#define HID_AUTH_FIELD_CUSTOM5                                  "SW_HID_AUTH_FIELD_CUSTOM5"
#define HID_AUTH_FIELD_ISBN                                     "SW_HID_AUTH_FIELD_ISBN"

#define HID_INSERT_INDEX_ENTRY_LEVEL_LB                         "SW_HID_INSERT_INDEX_ENTRY_LEVEL_LB"
#define HID_DLG_CREATE_AUTOMARK                                 "SW_HID_DLG_CREATE_AUTOMARK"
#define HID_BUSINESS_CARD_CONTENT                               "SW_HID_BUSINESS_CARD_CONTENT"
#define HID_SAVE_LABEL_DLG                                      "SW_HID_SAVE_LABEL_DLG"
#define HID_BUSINESS_FMT_PAGE                                   "SW_HID_BUSINESS_FMT_PAGE"
#define HID_BUSINESS_FMT_PAGE_CONT                              "SW_HID_BUSINESS_FMT_PAGE_CONT"
#define HID_BUSINESS_FMT_PAGE_SHEET                             "SW_HID_BUSINESS_FMT_PAGE_SHEET"
#define HID_BUSINESS_FMT_PAGE_BRAND                             "SW_HID_BUSINESS_FMT_PAGE_BRAND"
#define HID_BUSINESS_FMT_PAGE_TYPE                              "SW_HID_BUSINESS_FMT_PAGE_TYPE"
#define HID_SEND_MASTER_DIALOG                                  "SW_HID_SEND_MASTER_DIALOG"
#define HID_SEND_MASTER_CTRL_PUSHBUTTON_OK                      "SW_HID_SEND_MASTER_CTRL_PUSHBUTTON_OK"
#define HID_SEND_MASTER_CTRL_PUSHBUTTON_CANCEL                  "SW_HID_SEND_MASTER_CTRL_PUSHBUTTON_CANCEL"
#define HID_SEND_MASTER_CTRL_LISTBOX_FILTER                     "SW_HID_SEND_MASTER_CTRL_LISTBOX_FILTER"
#define HID_SEND_MASTER_CTRL_CONTROL_FILEVIEW                   "SW_HID_SEND_MASTER_CTRL_CONTROL_FILEVIEW"
#define HID_SEND_MASTER_CTRL_EDIT_FILEURL                       "SW_HID_SEND_MASTER_CTRL_EDIT_FILEURL"
#define HID_SEND_MASTER_CTRL_CHECKBOX_AUTOEXTENSION             "SW_HID_SEND_MASTER_CTRL_CHECKBOX_AUTOEXTENSION"
#define HID_SEND_MASTER_CTRL_LISTBOX_TEMPLATE                   "SW_HID_SEND_MASTER_CTRL_LISTBOX_TEMPLATE"

#define HID_SEND_HTML_DIALOG                                    "SW_HID_SEND_HTML_DIALOG"
#define HID_SEND_HTML_CTRL_PUSHBUTTON_OK                        "SW_HID_SEND_HTML_CTRL_PUSHBUTTON_OK"
#define HID_SEND_HTML_CTRL_PUSHBUTTON_CANCEL                    "SW_HID_SEND_HTML_CTRL_PUSHBUTTON_CANCEL"
#define HID_SEND_HTML_CTRL_LISTBOX_FILTER                       "SW_HID_SEND_HTML_CTRL_LISTBOX_FILTER"
#define HID_SEND_HTML_CTRL_CONTROL_FILEVIEW                     "SW_HID_SEND_HTML_CTRL_CONTROL_FILEVIEW"
#define HID_SEND_HTML_CTRL_EDIT_FILEURL                         "SW_HID_SEND_HTML_CTRL_EDIT_FILEURL"
#define HID_SEND_HTML_CTRL_CHECKBOX_AUTOEXTENSION               "SW_HID_SEND_HTML_CTRL_CHECKBOX_AUTOEXTENSION"
#define HID_SEND_HTML_CTRL_LISTBOX_TEMPLATE                     "SW_HID_SEND_HTML_CTRL_LISTBOX_TEMPLATE"

#define HID_COLUMN_VALUESET                                     "SW_HID_COLUMN_VALUESET"
#define HID_PVIEW_ZOOM_LB                                       "SW_HID_PVIEW_ZOOM_LB"
#define HID_MAIL_MERGE_SELECT                                   "SW_HID_MAIL_MERGE_SELECT"
#define HID_MAIL_MERGE_CREATE_FROM                              "SW_HID_MAIL_MERGE_CREATE_FROM"
#define HID_MAIL_MERGE_INSERT_FIELDS                            "SW_HID_MAIL_MERGE_INSERT_FIELDS"
#define HID_PRINT_AS_MERGE                                      "SW_HID_PRINT_AS_MERGE"
#define HID_MERGE_SOURCE_UNAVAILABLE                            "SW_HID_MERGE_SOURCE_UNAVAILABLE"
#define HID_DATA_SOURCES_UNAVAILABLE                            "SW_HID_DATA_SOURCES_UNAVAILABLE"
#define HID_MODULE_TOOLBOX                                      "SW_HID_MODULE_TOOLBOX"
#define HID_SECTION_INDENTS_PAGE                                "SW_HID_SECTION_INDENTS_PAGE"

#define HID_FLD_DROPDOWN                                        "SW_HID_FLD_DROPDOWN"
#define HID_MM_DOCSELECTPAGE                                    "SW_HID_MM_DOCSELECTPAGE"
#define HID_MM_OUTPUTTYPEPAGE                                   "SW_HID_MM_OUTPUTTYPEPAGE"
#define HID_MM_ADDRESSBLOCKPAGE                                 "SW_HID_MM_ADDRESSBLOCKPAGE"
#define HID_MM_GREETINGSPAGE                                    "SW_HID_MM_GREETINGSPAGE"
#define HID_MM_PREPAREMERGEPAGE                                 "SW_HID_MM_PREPAREMERGEPAGE"
#define HID_MM_MERGEPAGE                                        "SW_HID_MM_MERGEPAGE"
#define HID_MM_OUTPUTPAGE                                       "SW_HID_MM_OUTPUTPAGE"
#define HID_MM_ADDRESSLISTDIALOG                                "SW_HID_MM_ADDRESSLISTDIALOG"
#define HID_MM_SELECTDBTABLEDDIALOG                             "SW_HID_MM_SELECTDBTABLEDDIALOG"
#define HID_MM_SELECTDBTABLEDDIALOG_LISTBOX                     "SW_HID_MM_SELECTDBTABLEDDIALOG_LISTBOX"
#define HID_MM_DBTABLEPREVIEWDIALOG                             "SW_HID_MM_DBTABLEPREVIEWDIALOG"
#define HID_MM_CREATEADDRESSLIST                                "SW_HID_MM_CREATEADDRESSLIST"
#define HID_MM_FIND_ENTRY                                       "SW_HID_MM_FIND_ENTRY"
#define HID_MM_ADD_RENAME_ENTRY                                 "SW_HID_MM_ADD_RENAME_ENTRY"
#define HID_MM_CUSTOMIZE_ADDRESS_LIST                           "SW_HID_MM_CUSTOMIZE_ADDRESS_LIST"
#define HID_MM_CUSTOMIZEADDRESSBLOCK                            "SW_HID_MM_CUSTOMIZEADDRESSBLOCK"
#define HID_MM_SELECTADDRESSBLOCK                               "SW_HID_MM_SELECTADDRESSBLOCK"
#define HID_MM_ASSIGNFIELDS                                     "SW_HID_MM_ASSIGNFIELDS"
#define HID_MM_LAYOUT_PAGE                                      "SW_HID_MM_LAYOUT_PAGE"
#define HID_MM_COPYTO                                           "SW_HID_MM_COPYTO"
#define HID_MM_ADDRESSLIST_HB                                   "SW_HID_MM_ADDRESSLIST_HB"
#define HID_MM_ADDRESSLIST_TLB                                  "SW_HID_MM_ADDRESSLIST_TLB"
#define HID_TP_MAILCONFIG                                       "SW_HID_TP_MAILCONFIG"
#define HID_MM_TESTACCOUNTSETTINGS                              "SW_HID_MM_TESTACCOUNTSETTINGS"
#define HID_MM_TESTACCOUNTSETTINGS_TLB                          "SW_HID_MM_TESTACCOUNTSETTINGS_TLB"
#define HID_MM_TESTACCOUNTSETTINGS_HB                           "SW_HID_MM_TESTACCOUNTSETTINGS_HB"
#define HID_MM_MAILBODY                                         "SW_HID_MM_MAILBODY"
#define HID_MM_SENDMAILS                                        "SW_HID_MM_SENDMAILS"
#define HID_MM_SENDMAILS_STATUSLB                               "SW_HID_MM_SENDMAILS_STATUSLB"
#define HID_MAILMERGECHILD                                      "SW_HID_MAILMERGECHILD"
#define HID_MM_CUSTOMFIELDS                                     "SW_HID_MM_CUSTOMFIELDS"
#define HID_MM_MAILSTATUS_TLB                                   "SW_HID_MM_MAILSTATUS_TLB"
#define HID_RETURN_TO_MAILMERGE                                 "SW_HID_RETURN_TO_MAILMERGE"
#define HID_MM_SAVEWARNING                                      "SW_HID_MM_SAVEWARNING"
#define HID_MM_QUERY                                            "SW_HID_MM_QUERY"
#define HID_MM_SENDWARNING                                      "SW_HID_MM_SENDWARNING"
#define HID_MM_SERVERAUTHENTICATION                             "SW_HID_MM_SERVERAUTHENTICATION"

#define HID_NID_TBL                                             "SW_HID_NID_TBL"
#define HID_NID_FRM                                             "SW_HID_NID_FRM"
#define HID_NID_GRF                                             "SW_HID_NID_GRF"
#define HID_NID_OLE                                             "SW_HID_NID_OLE"
#define HID_NID_PGE                                             "SW_HID_NID_PGE"
#define HID_NID_OUTL                                            "SW_HID_NID_OUTL"
#define HID_NID_MARK                                            "SW_HID_NID_MARK"
#define HID_NID_DRW                                             "SW_HID_NID_DRW"
#define HID_NID_CTRL                                            "SW_HID_NID_CTRL"
#define HID_NID_PREV                                            "SW_HID_NID_PREV"
#define HID_NID_REG                                             "SW_HID_NID_REG"
#define HID_NID_BKM                                             "SW_HID_NID_BKM"
#define HID_NID_SEL                                             "SW_HID_NID_SEL"
#define HID_NID_FTN                                             "SW_HID_NID_FTN"
#define HID_NID_POSTIT                                          "SW_HID_NID_POSTIT"
#define HID_NID_SRCH_REP                                        "SW_HID_NID_SRCH_REP"
#define HID_NID_INDEX_ENTRY                                     "SW_HID_NID_INDEX_ENTRY"
#define HID_NID_TABLE_FORMULA                                   "SW_HID_NID_TABLE_FORMULA"
#define HID_NID_TABLE_FORMULA_ERROR                             "SW_HID_NID_TABLE_FORMULA_ERROR"
#define HID_NID_NEXT                                            "SW_HID_NID_NEXT"
#define HID_MM_NEXT_PAGE                                        "SW_HID_MM_NEXT_PAGE"
#define HID_MM_PREV_PAGE                                        "SW_HID_MM_PREV_PAGE"
#define HID_MM_GREETINGS_CB_PERSONALIZED                        "SW_HID_MM_GREETINGS_CB_PERSONALIZED"
#define HID_MM_GREETINGS_LB_FEMALE                              "SW_HID_MM_GREETINGS_LB_FEMALE"
#define HID_MM_GREETINGS_PB_FEMALE                              "SW_HID_MM_GREETINGS_PB_FEMALE"
#define HID_MM_GREETINGS_LB_MALE                                "SW_HID_MM_GREETINGS_LB_MALE"
#define HID_MM_GREETINGS_PB_MALE                                "SW_HID_MM_GREETINGS_PB_MALE"
#define HID_MM_GREETINGS_LB_FEMALECOLUMN                        "SW_HID_MM_GREETINGS_LB_FEMALECOLUMN"
#define HID_MM_GREETINGS_CB_FEMALEFIELD                         "SW_HID_MM_GREETINGS_CB_FEMALEFIELD"
#define HID_MM_GREETINGS_CB_NEUTRAL                             "SW_HID_MM_GREETINGS_CB_NEUTRAL"
#define HID_MM_BODY_CB_PERSONALIZED                             "SW_HID_MM_BODY_CB_PERSONALIZED"
#define HID_MM_BODY_LB_FEMALE                                   "SW_HID_MM_BODY_LB_FEMALE"
#define HID_MM_BODY_PB_FEMALE                                   "SW_HID_MM_BODY_PB_FEMALE"
#define HID_MM_BODY_LB_MALE                                     "SW_HID_MM_BODY_LB_MALE"
#define HID_MM_BODY_PB_MALE                                     "SW_HID_MM_BODY_PB_MALE"
#define HID_MM_BODY_LB_FEMALECOLUMN                             "SW_HID_MM_BODY_LB_FEMALECOLUMN"
#define HID_MM_BODY_CB_FEMALEFIELD                              "SW_HID_MM_BODY_CB_FEMALEFIELD"
#define HID_MM_BODY_CB_NEUTRAL                                  "SW_HID_MM_BODY_CB_NEUTRAL"
#define HID_MM_ASSIGN_FIELDS                                    "SW_HID_MM_ASSIGN_FIELDS"
#define HID_MM_HEADER_0                                         "SW_HID_MM_HEADER_0"
#define HID_MM_HEADER_1                                         "SW_HID_MM_HEADER_1"
#define HID_MM_HEADER_2                                         "SW_HID_MM_HEADER_2"
#define HID_MM_HEADER_3                                         "SW_HID_MM_HEADER_3"
#define HID_MM_HEADER_4                                         "SW_HID_MM_HEADER_4"
#define HID_MM_HEADER_5                                         "SW_HID_MM_HEADER_5"
#define HID_MM_HEADER_6                                         "SW_HID_MM_HEADER_6"
#define HID_MM_HEADER_7                                         "SW_HID_MM_HEADER_7"
#define HID_MM_HEADER_8                                         "SW_HID_MM_HEADER_8"
#define HID_MM_HEADER_9                                         "SW_HID_MM_HEADER_9"
#define HID_MM_HEADER_10                                        "SW_HID_MM_HEADER_10"
#define HID_MM_HEADER_11                                        "SW_HID_MM_HEADER_11"
#define HID_MM_HEADER_12                                        "SW_HID_MM_HEADER_12"
#define HID_MM_HEADER_13                                        "SW_HID_MM_HEADER_13"

#define HID_ITM_HOR_LEFT                                        "SW_HID_ITM_HOR_LEFT"
#define HID_ITM_HOR_CNTR                                        "SW_HID_ITM_HOR_CNTR"
#define HID_ITM_HOR_RGHT                                        "SW_HID_ITM_HOR_RGHT"
#define HID_ITM_VER_LEFT                                        "SW_HID_ITM_VER_LEFT"
#define HID_ITM_VER_CNTR                                        "SW_HID_ITM_VER_CNTR"
#define HID_ITM_VER_RGHT                                        "SW_HID_ITM_VER_RGHT"

#define HID_TBX_FORMULA_CALC                                    "SW_HID_TBX_FORMULA_CALC"
#define HID_TBX_FORMULA_CANCEL                                  "SW_HID_TBX_FORMULA_CANCEL"
#define HID_TBX_FORMULA_APPLY                                   "SW_HID_TBX_FORMULA_APPLY"

#define HID_TITLEPAGE                                           "SW_HID_TITLEPAGE"

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
